<?php
namespace app\attachment\controller;

use app\common\controller\Front;

class Index extends Front
{
    public function _initialize()
    {
        $this->request->filter('trim,strip_tags,htmlspecialchars');
        
        parent::_initialize();
    }
    
    //按path查询
    public function index()
    {
        //ID参数
        if(!$val = input('val/s')){
            $this->error(lang('mustIn'));
        }
        //查询数据
        $info = model('common/Info','loglic')->get([
            'cache'   => true,
            'status'  => 'normal',
            'module'  => 'attachment',
            'with'    => 'info_meta',
            'excerpt' => $val,
        ]);
        //公共入口
        $this->detail($info);
    }
    
    //按ID查询
    public function id()
    {
        //ID参数
        if(!$val = input('val/d',0)){
            $this->error(lang('mustIn'));
        }
        //查询数据
        $info = model('common/Info','loglic')->get([
            'cache'  => true,
            'status' => 'normal',
            'module' => 'attachment',
            'with'   => 'info_meta',
            'id'     => $val,
        ]);
        //公共入口
        $this->detail($info);
    }
    
    //按slug查询
    public function slug()
    {
        //ID参数
        if(!$val = input('val/s')){
            $this->error(lang('mustIn'));
        }
        //查询数据
        $info = model('common/Info','loglic')->get([
            'cache'  => true,
            'status' => 'normal',
            'module' => 'attachment',
            'with'   => 'info_meta',
            'slug'   => $val,
        ]);
        //公共入口
        $this->detail($info);
    }
    
    //公用detail
    private function detail($info=[])
    {
        if(!$info){
            $this->error('无数据');
        }
        if(!$info['info_mime_type']){
            $this->error('文件类型不正确');
        }
        //远程附件显示到浏览器
        if( config('common.upload_cdn') ){
            $this->curlBrowser($info['info_excerpt'], $info['info_mime_type'], $info['info_title']);
        }
        //本地附件显示到浏览器
        $this->sendBrowser($info['info_excerpt'], $info['info_mime_type'], $info['info_title']);
    }
    
    //发送本地文件到客户端
    private function sendBrowser($filePath='', $mimeType='image/png', $fileName='test.zip')
    {
        //原始附件地址
        $filePath = trim(config('common.upload_path')).'/'.$filePath;
        //判断本地文件
        if(!file_exists($filePath)){
            $this->error('文件已失效，请联系管理员');
        }
        if(!is_readable($filePath)){
            $this->error('无读取权限，请联系管理员');
        }
        //HEADER
        header('Content-Type: '.$mimeType);//application/octet-stream
        header('Content-Length: ' . filesize($filePath));
        //保存文件名
        if(substr($mimeType,0,5) != 'image'){
            header('Content-Disposition: attachment;filename = '.$fileName);//basename($file)
        }
        //读取文件
        exit(readfile($filePath));
    }
    
    //发送远程附件到浏览器
    private function curlBrowser($url='', $mimeType='image/png', $fileName='')
    {
        //是否远程URL
        if( !parse_url($url, PHP_URL_SCHEME) ){
            $url = rtrim(config('common.upload_cdn'),'/').$this->site['path_root'].config('common.upload_path').'/'.$url;
        }
        //CURL连接
        $ch = curl_init();
        curl_setopt ($ch, CURLOPT_URL, $url);
        curl_setopt ($ch, CURLOPT_HEADER, 0);
        curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1); 
        curl_setopt ($ch, CURLOPT_TIMEOUT, 10);
        curl_setopt ($ch, CURLOPT_CONNECTTIMEOUT,10);
        curl_setopt ($ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt ($ch, CURLOPT_ENCODING, "");
        //https自动处理
        if(parse_url($url, PHP_URL_SCHEME) == 'https'){
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
        }
        //获取到的内容
        $content = curl_exec($ch);
        //关闭链接
        curl_close($ch);
        //验证数据
        if(!$content){
            $this->error('获取远程附件出错');
        }
        //HEADER
        header('Content-Type: '.$mimeType);
        //保存文件名
        if(substr($mimeType,0,5) != 'image'){
            header('Content-Disposition: attachment;filename = '.DcEmpty($fileName, 'down'));
        }
        //显示内容
        exit($content);
    }
}